@@ -30,6 +30,7 @@ use std::sync::Arc;
 /// # Example
 ///
 /// ```
+/// # use std::sync::Arc;
 /// # use arrow_array::{Array, FixedSizeListArray, Int32Array};
 /// # use arrow_data::ArrayData;
 /// # use arrow_schema::{DataType, Field};
@@ -41,7 +42,7 @@ use std::sync::Arc;
 ///     .build()
 ///     .unwrap();
 /// let list_data_type = DataType::FixedSizeList(
-///     Box::new(Field::new("item", DataType::Int32, false)),
+///     Arc::new(Field::new("item", DataType::Int32, false)),
 ///     3,
 /// );
 /// let list_data = ArrayData::builder(list_data_type.clone())
@@ -106,6 +107,12 @@ impl FixedSizeListArray {
         i as i32 * self.length
     }
 
+    /// Returns a zero-copy slice of this array with the indicated offset and length.
+    pub fn slice(&self, offset: usize, length: usize) -> Self {
+        // TODO: Slice buffers directly (#3880)
+        self.data.slice(offset, length).into()
+    }
+
     /// Creates a [`FixedSizeListArray`] from an iterator of primitive values
     /// # Example
     /// ```
@@ -216,8 +223,7 @@ impl Array for FixedSizeListArray {
     }
 
     fn slice(&self, offset: usize, length: usize) -> ArrayRef {
-        // TODO: Slice buffers directly (#3880)
-        Arc::new(Self::from(self.data.slice(offset, length)))
+        Arc::new(self.slice(offset, length))
     }
 
     fn nulls(&self) -> Option<&NullBuffer> {
@@ -250,6 +256,8 @@ impl std::fmt::Debug for FixedSizeListArray {
 #[cfg(test)]
 mod tests {
     use super::*;
+    use crate::cast::AsArray;
+    use crate::types::Int32Type;
     use crate::Int32Array;
     use arrow_buffer::{bit_util, Buffer};
     use arrow_schema::Field;
@@ -265,7 +273,7 @@ mod tests {
 
         // Construct a list array from the above two
         let list_data_type = DataType::FixedSizeList(
-            Box::new(Field::new("item", DataType::Int32, false)),
+            Arc::new(Field::new("item", DataType::Int32, false)),
             3,
         );
         let list_data = ArrayData::builder(list_data_type.clone())
@@ -275,7 +283,7 @@ mod tests {
             .unwrap();
         let list_array = FixedSizeListArray::from(list_data);
 
-        assert_eq!(&value_data, list_array.values().data());
+        assert_eq!(value_data, list_array.values().to_data());
         assert_eq!(DataType::Int32, list_array.value_type());
         assert_eq!(3, list_array.len());
         assert_eq!(0, list_array.null_count());
@@ -304,19 +312,11 @@ mod tests {
             .unwrap();
         let list_array = FixedSizeListArray::from(list_data);
 
-        assert_eq!(&value_data, list_array.values().data());
+        assert_eq!(value_data, list_array.values().to_data());
         assert_eq!(DataType::Int32, list_array.value_type());
         assert_eq!(3, list_array.len());
         assert_eq!(0, list_array.null_count());
-        assert_eq!(
-            3,
-            list_array
-                .value(0)
-                .as_any()
-                .downcast_ref::<Int32Array>()
-                .unwrap()
-                .value(0)
-        );
+        assert_eq!(3, list_array.value(0).as_primitive::<Int32Type>().value(0));
         assert_eq!(6, list_array.value_offset(1));
         assert_eq!(3, list_array.value_length());
     }
@@ -338,7 +338,7 @@ mod tests {
 
         // Construct a list array from the above two
         let list_data_type = DataType::FixedSizeList(
-            Box::new(Field::new("item", DataType::Int32, false)),
+            Arc::new(Field::new("item", DataType::Int32, false)),
             3,
         );
         let list_data = unsafe {
@@ -369,7 +369,7 @@ mod tests {
 
         // Construct a fixed size list array from the above two
         let list_data_type = DataType::FixedSizeList(
-            Box::new(Field::new("item", DataType::Int32, false)),
+            Arc::new(Field::new("item", DataType::Int32, false)),
             2,
         );
         let list_data = ArrayData::builder(list_data_type)
@@ -380,7 +380,7 @@ mod tests {
             .unwrap();
         let list_array = FixedSizeListArray::from(list_data);
 
-        assert_eq!(&value_data, list_array.values().data());
+        assert_eq!(value_data, list_array.values().to_data());
         assert_eq!(DataType::Int32, list_array.value_type());
         assert_eq!(5, list_array.len());
         assert_eq!(2, list_array.null_count());
@@ -430,7 +430,7 @@ mod tests {
 
         // Construct a fixed size list array from the above two
         let list_data_type = DataType::FixedSizeList(
-            Box::new(Field::new("item", DataType::Int32, false)),
+            Arc::new(Field::new("item", DataType::Int32, false)),
             2,
         );
         let list_data = ArrayData::builder(list_data_type)
