@@ -15,15 +15,37 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use crate::{make_array, Array, ArrayRef};
+use crate::{make_array, Array, ArrayRef, RecordBatch};
 use arrow_buffer::{buffer_bin_or, Buffer, NullBuffer};
 use arrow_data::ArrayData;
-use arrow_schema::{ArrowError, DataType, Field};
+use arrow_schema::{ArrowError, DataType, Field, Fields, SchemaBuilder};
 use std::sync::Arc;
 use std::{any::Any, ops::Index};
 
 /// A nested array type where each child (called *field*) is represented by a separate
 /// array.
+///
+///
+/// # Comparison with [RecordBatch]
+///
+/// Both [`RecordBatch`] and [`StructArray`] represent a collection of columns / arrays with the
+/// same length.
+///
+/// However, there are a couple of key differences:
+///
+/// * [`StructArray`] can be nested within other [`Array`], including itself
+/// * [`RecordBatch`] can contain top-level metadata on its associated [`Schema`][arrow_schema::Schema]
+/// * [`StructArray`] can contain top-level nulls, i.e. `null`
+/// * [`RecordBatch`] can only represent nulls in its child columns, i.e. `{"field": null}`
+///
+/// [`StructArray`] is therefore a more general data container than [`RecordBatch`], and as such
+/// code that needs to handle both will typically share an implementation in terms of
+/// [`StructArray`] and convert to/from [`RecordBatch`] as necessary.
+///
+/// [`From`] implementations are provided to facilitate this conversion, however, converting
+/// from a [`StructArray`] containing top-level nulls to a [`RecordBatch`] will panic, as there
+/// is no way to preserve them.
+///
 /// # Example: Create an array from a vector of fields
 ///
 /// ```
@@ -89,6 +111,14 @@ impl StructArray {
         }
     }
 
+    /// Returns the [`Fields`] of this [`StructArray`]
+    pub fn fields(&self) -> &Fields {
+        match self.data_type() {
+            DataType::Struct(f) => f,
+            _ => unreachable!(),
+        }
+    }
+
     /// Return child array whose field name equals to column_name
     ///
     /// Note: A schema can currently have duplicate field names, in which case
@@ -100,6 +130,12 @@ impl StructArray {
             .position(|c| c == &column_name)
             .map(|pos| self.column(pos))
     }
+
+    /// Returns a zero-copy slice of this array with the indicated offset and length.
+    pub fn slice(&self, offset: usize, length: usize) -> Self {
+        // TODO: Slice buffers directly (#3880)
+        self.data.slice(offset, length).into()
+    }
 }
 
 impl From<ArrayData> for StructArray {
@@ -138,7 +174,7 @@ impl TryFrom<Vec<(&str, ArrayRef)>> for StructArray {
         // null: the null mask of the arrays.
         let mut null: Option<Buffer> = None;
         for (field_name, array) in values {
-            let child_datum = array.data();
+            let child_datum = array.to_data();
             let child_datum_len = child_datum.len();
             if let Some(len) = len {
                 if len != child_datum_len {
@@ -150,12 +186,11 @@ impl TryFrom<Vec<(&str, ArrayRef)>> for StructArray {
             } else {
                 len = Some(child_datum_len)
             }
-            child_data.push(child_datum.clone());
-            fields.push(Field::new(
+            fields.push(Arc::new(Field::new(
                 field_name,
                 array.data_type().clone(),
                 child_datum.nulls().is_some(),
-            ));
+            )));
 
             if let Some(child_nulls) = child_datum.nulls() {
                 null = Some(if let Some(null_buffer) = &null {
@@ -173,10 +208,11 @@ impl TryFrom<Vec<(&str, ArrayRef)>> for StructArray {
                 // when one of the fields has no nulls, then there is no null in the array
                 null = None;
             }
+            child_data.push(child_datum);
         }
         let len = len.unwrap();
 
-        let builder = ArrayData::builder(DataType::Struct(fields))
+        let builder = ArrayData::builder(DataType::Struct(fields.into()))
             .len(len)
             .null_bit_buffer(null)
             .child_data(child_data);
@@ -205,8 +241,7 @@ impl Array for StructArray {
     }
 
     fn slice(&self, offset: usize, length: usize) -> ArrayRef {
-        // TODO: Slice buffers directly (#3880)
-        Arc::new(Self::from(self.data.slice(offset, length)))
+        Arc::new(self.slice(offset, length))
     }
 
     fn nulls(&self) -> Option<&NullBuffer> {
@@ -216,29 +251,32 @@ impl Array for StructArray {
 
 impl From<Vec<(Field, ArrayRef)>> for StructArray {
     fn from(v: Vec<(Field, ArrayRef)>) -> Self {
-        let (field_types, field_values): (Vec<_>, Vec<_>) = v.into_iter().unzip();
-
-        let length = field_values.get(0).map(|a| a.len()).unwrap_or(0);
-        field_types.iter().zip(field_values.iter()).for_each(
-            |(field_type, field_value)| {
-                // Check the length of the child arrays
-                assert_eq!(
-                    length,
-                    field_value.len(),
-                    "all child arrays of a StructArray must have the same length"
-                );
-                // Check data types of child arrays
-                assert_eq!(
-                    field_type.data_type(),
-                    field_value.data().data_type(),
-                    "the field data types must match the array data in a StructArray"
-                );
-            },
-        );
+        let iter = v.into_iter();
+        let capacity = iter.size_hint().0;
 
+        let mut len = None;
+        let mut schema = SchemaBuilder::with_capacity(capacity);
+        let mut child_data = Vec::with_capacity(capacity);
+        for (field, array) in iter {
+            // Check the length of the child arrays
+            assert_eq!(
+                *len.get_or_insert(array.len()),
+                array.len(),
+                "all child arrays of a StructArray must have the same length"
+            );
+            // Check data types of child arrays
+            assert_eq!(
+                field.data_type(),
+                array.data_type(),
+                "the field data types must match the array data in a StructArray"
+            );
+            schema.push(field);
+            child_data.push(array.to_data());
+        }
+        let field_types = schema.finish().fields;
         let array_data = ArrayData::builder(DataType::Struct(field_types))
-            .child_data(field_values.into_iter().map(|a| a.into_data()).collect())
-            .len(length);
+            .child_data(child_data)
+            .len(len.unwrap_or_default());
         let array_data = unsafe { array_data.build_unchecked() };
 
         // We must validate nullability
@@ -269,30 +307,31 @@ impl std::fmt::Debug for StructArray {
 
 impl From<(Vec<(Field, ArrayRef)>, Buffer)> for StructArray {
     fn from(pair: (Vec<(Field, ArrayRef)>, Buffer)) -> Self {
-        let (field_types, field_values): (Vec<_>, Vec<_>) = pair.0.into_iter().unzip();
-
-        let length = field_values.get(0).map(|a| a.len()).unwrap_or(0);
-        field_types.iter().zip(field_values.iter()).for_each(
-            |(field_type, field_value)| {
-                // Check the length of the child arrays
-                assert_eq!(
-                    length,
-                    field_value.len(),
-                    "all child arrays of a StructArray must have the same length"
-                );
-                // Check data types of child arrays
-                assert_eq!(
-                    field_type.data_type(),
-                    field_value.data().data_type(),
-                    "the field data types must match the array data in a StructArray"
-                );
-            },
-        );
-
+        let capacity = pair.0.len();
+        let mut len = None;
+        let mut schema = SchemaBuilder::with_capacity(capacity);
+        let mut child_data = Vec::with_capacity(capacity);
+        for (field, array) in pair.0 {
+            // Check the length of the child arrays
+            assert_eq!(
+                *len.get_or_insert(array.len()),
+                array.len(),
+                "all child arrays of a StructArray must have the same length"
+            );
+            // Check data types of child arrays
+            assert_eq!(
+                field.data_type(),
+                array.data_type(),
+                "the field data types must match the array data in a StructArray"
+            );
+            schema.push(field);
+            child_data.push(array.to_data());
+        }
+        let field_types = schema.finish().fields;
         let array_data = ArrayData::builder(DataType::Struct(field_types))
             .null_bit_buffer(Some(pair.1))
-            .child_data(field_values.into_iter().map(|a| a.into_data()).collect())
-            .len(length);
+            .child_data(child_data)
+            .len(len.unwrap_or_default());
         let array_data = unsafe { array_data.build_unchecked() };
 
         // We must validate nullability
@@ -302,6 +341,21 @@ impl From<(Vec<(Field, ArrayRef)>, Buffer)> for StructArray {
     }
 }
 
+impl From<RecordBatch> for StructArray {
+    fn from(value: RecordBatch) -> Self {
+        // TODO: Don't store ArrayData inside arrays (#3880)
+        let builder = ArrayData::builder(DataType::Struct(value.schema().fields.clone()))
+            .child_data(value.columns().iter().map(|x| x.to_data()).collect())
+            .len(value.num_rows());
+
+        // Safety: RecordBatch must be valid
+        Self {
+            data: unsafe { builder.build_unchecked() },
+            boxed_fields: value.columns().to_vec(),
+        }
+    }
+}
+
 impl Index<&str> for StructArray {
     type Output = ArrayRef;
 
@@ -331,25 +385,23 @@ mod tests {
 
     #[test]
     fn test_struct_array_builder() {
-        let array = BooleanArray::from(vec![false, false, true, true]);
-        let boolean_data = array.data();
-        let array = Int64Array::from(vec![42, 28, 19, 31]);
-        let int_data = array.data();
+        let boolean_array = BooleanArray::from(vec![false, false, true, true]);
+        let int_array = Int64Array::from(vec![42, 28, 19, 31]);
 
         let fields = vec![
             Field::new("a", DataType::Boolean, false),
             Field::new("b", DataType::Int64, false),
         ];
-        let struct_array_data = ArrayData::builder(DataType::Struct(fields))
+        let struct_array_data = ArrayData::builder(DataType::Struct(fields.into()))
             .len(4)
-            .add_child_data(boolean_data.clone())
-            .add_child_data(int_data.clone())
+            .add_child_data(boolean_array.to_data())
+            .add_child_data(int_array.to_data())
             .build()
             .unwrap();
         let struct_array = StructArray::from(struct_array_data);
 
-        assert_eq!(boolean_data, struct_array.column(0).data());
-        assert_eq!(int_data, struct_array.column(1).data());
+        assert_eq!(struct_array.column(0).as_ref(), &boolean_array);
+        assert_eq!(struct_array.column(1).as_ref(), &int_array);
     }
 
     #[test]
@@ -410,7 +462,7 @@ mod tests {
             StructArray::try_from(vec![("f1", strings.clone()), ("f2", ints.clone())])
                 .unwrap();
 
-        let struct_data = arr.data();
+        let struct_data = arr.into_data();
         assert_eq!(4, struct_data.len());
         assert_eq!(1, struct_data.null_count());
         assert_eq!(
@@ -434,8 +486,8 @@ mod tests {
             .build()
             .unwrap();
 
-        assert_eq!(expected_string_data, *arr.column(0).data());
-        assert_eq!(expected_int_data, *arr.column(1).data());
+        assert_eq!(expected_string_data, struct_data.child_data()[0]);
+        assert_eq!(expected_int_data, struct_data.child_data()[1]);
     }
 
     #[test]
@@ -509,7 +561,7 @@ mod tests {
             Field::new("a", DataType::Boolean, true),
             Field::new("b", DataType::Int32, true),
         ];
-        let struct_array_data = ArrayData::builder(DataType::Struct(field_types))
+        let struct_array_data = ArrayData::builder(DataType::Struct(field_types.into()))
             .len(5)
             .add_child_data(boolean_data.clone())
             .add_child_data(int_data.clone())
@@ -525,8 +577,8 @@ mod tests {
         assert!(struct_array.is_valid(2));
         assert!(struct_array.is_null(3));
         assert!(struct_array.is_valid(4));
-        assert_eq!(&boolean_data, struct_array.column(0).data());
-        assert_eq!(&int_data, struct_array.column(1).data());
+        assert_eq!(boolean_data, struct_array.column(0).to_data());
+        assert_eq!(int_data, struct_array.column(1).to_data());
 
         let c0 = struct_array.column(0);
         let c0 = c0.as_any().downcast_ref::<BooleanArray>().unwrap();
