@@ -240,7 +240,7 @@ mod tests {
         }
 
         let arr = builder.finish();
-        assert_eq!(&buf, arr.values());
+        assert_eq!(&buf, arr.values().inner());
         assert_eq!(10, arr.len());
         assert_eq!(0, arr.offset());
         assert_eq!(0, arr.null_count());
@@ -289,7 +289,7 @@ mod tests {
 
         let array = builder.finish();
         assert_eq!(0, array.null_count());
-        assert!(array.data().nulls().is_none());
+        assert!(array.nulls().is_none());
     }
 
     #[test]
@@ -311,7 +311,7 @@ mod tests {
         assert_eq!(4, array.false_count());
 
         assert_eq!(0, array.null_count());
-        assert!(array.data().nulls().is_none());
+        assert!(array.nulls().is_none());
     }
 
     #[test]
