@@ -16,9 +16,28 @@
 // under the License.
 
 use num::cast::AsPrimitive;
-use num::{BigInt, FromPrimitive, Num, ToPrimitive};
+use num::{BigInt, FromPrimitive, ToPrimitive};
 use std::cmp::Ordering;
-use std::ops::{BitAnd, BitOr, BitXor, Shl, Shr};
+use std::num::ParseIntError;
+use std::ops::{BitAnd, BitOr, BitXor, Neg, Shl, Shr};
+use std::str::FromStr;
+
+/// An opaque error similar to [`std::num::ParseIntError`]
+#[derive(Debug)]
+pub struct ParseI256Error {}
+
+impl From<ParseIntError> for ParseI256Error {
+    fn from(_: ParseIntError) -> Self {
+        Self {}
+    }
+}
+
+impl std::fmt::Display for ParseI256Error {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "Failed to parse as i256")
+    }
+}
+impl std::error::Error for ParseI256Error {}
 
 /// A signed 256-bit integer
 #[allow(non_camel_case_types)]
@@ -40,6 +59,67 @@ impl std::fmt::Display for i256 {
     }
 }
 
+impl FromStr for i256 {
+    type Err = ParseI256Error;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        // i128 can store up to 38 decimal digits
+        if s.len() <= 38 {
+            return Ok(Self::from_i128(i128::from_str(s)?));
+        }
+
+        let (negative, s) = match s.as_bytes()[0] {
+            b'-' => (true, &s[1..]),
+            b'+' => (false, &s[1..]),
+            _ => (false, s),
+        };
+
+        // Trim leading 0s
+        let s = s.trim_start_matches('0');
+        if s.is_empty() {
+            return Ok(i256::ZERO);
+        }
+
+        if !s.as_bytes()[0].is_ascii_digit() {
+            // Ensures no duplicate sign
+            return Err(ParseI256Error {});
+        }
+
+        parse_impl(s, negative)
+    }
+}
+
+/// Parse `s` with any sign and leading 0s removed
+fn parse_impl(s: &str, negative: bool) -> Result<i256, ParseI256Error> {
+    if s.len() <= 38 {
+        let low = i128::from_str(s)?;
+        return Ok(match negative {
+            true => i256::from_parts(low.neg() as _, -1),
+            false => i256::from_parts(low as _, 0),
+        });
+    }
+
+    let split = s.len() - 38;
+    if !s.as_bytes()[split].is_ascii_digit() {
+        // Ensures not splitting codepoint and no sign
+        return Err(ParseI256Error {});
+    }
+    let (hs, ls) = s.split_at(split);
+
+    let mut low = i128::from_str(ls)?;
+    let high = parse_impl(hs, negative)?;
+
+    if negative {
+        low = -low;
+    }
+
+    let low = i256::from_i128(low);
+
+    high.checked_mul(i256::from_i128(10_i128.pow(38)))
+        .and_then(|high| high.checked_add(low))
+        .ok_or(ParseI256Error {})
+}
+
 impl PartialOrd for i256 {
     fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
         Some(self.cmp(other))
@@ -106,14 +186,7 @@ impl i256 {
     /// Create an integer value from its representation as string.
     #[inline]
     pub fn from_string(value_str: &str) -> Option<Self> {
-        let numbers = BigInt::from_str_radix(value_str, 10).ok()?;
-        let (integer, overflow) = Self::from_bigint_with_overflow(numbers);
-
-        if overflow {
-            None
-        } else {
-            Some(integer)
-        }
+        value_str.parse().ok()
     }
 
     /// Create an optional i256 from the provided `f64`. Returning `None`
@@ -152,6 +225,11 @@ impl i256 {
         (high_negative == low_negative && high_valid).then_some(self.low as i128)
     }
 
+    /// Wraps this `i256` into an `i128`
+    pub fn as_i128(self) -> i128 {
+        self.low as i128
+    }
+
     /// Return the memory representation of this integer as a byte array in little-endian byte order.
     #[inline]
     pub const fn to_le_bytes(self) -> [u8; 32] {
@@ -243,9 +321,9 @@ impl i256 {
     /// Performs checked addition
     #[inline]
     pub fn checked_add(self, other: Self) -> Option<Self> {
-        let (low, carry) = self.low.overflowing_add(other.low);
-        let high = self.high.checked_add(other.high)?.checked_add(carry as _)?;
-        Some(Self { low, high })
+        let r = self.wrapping_add(other);
+        ((other.is_negative() && r < self) || (!other.is_negative() && r >= self))
+            .then_some(r)
     }
 
     /// Performs wrapping subtraction
@@ -259,9 +337,9 @@ impl i256 {
     /// Performs checked subtraction
     #[inline]
     pub fn checked_sub(self, other: Self) -> Option<Self> {
-        let (low, carry) = self.low.overflowing_sub(other.low);
-        let high = self.high.checked_sub(other.high)?.checked_sub(carry as _)?;
-        Some(Self { low, high })
+        let r = self.wrapping_sub(other);
+        ((other.is_negative() && r > self) || (!other.is_negative() && r <= self))
+            .then_some(r)
     }
 
     /// Performs wrapping multiplication
@@ -282,10 +360,14 @@ impl i256 {
     /// Performs checked multiplication
     #[inline]
     pub fn checked_mul(self, other: Self) -> Option<Self> {
+        if self == i256::ZERO || other == i256::ZERO {
+            return Some(i256::ZERO);
+        }
+
         // Shift sign bit down to construct mask of all set bits if negative
         let l_sa = self.high >> 127;
         let r_sa = other.high >> 127;
-        let out_sa = l_sa ^ r_sa;
+        let out_sa = (l_sa ^ r_sa) as u128;
 
         // Compute absolute values
         let l_abs = self.wrapping_abs();
@@ -303,13 +385,15 @@ impl i256 {
         let hl = (l_abs.high as u128).checked_mul(r_abs.low)?;
         let lh = l_abs.low.checked_mul(r_abs.high as u128)?;
 
-        let high: i128 = high.checked_add(hl)?.checked_add(lh)?.try_into().ok()?;
+        let high = high.checked_add(hl)?.checked_add(lh)?;
 
         // Reverse absolute value, if necessary
-        let (low, c) = (low ^ out_sa as u128).overflowing_sub(out_sa as u128);
-        let high = (high ^ out_sa).wrapping_sub(out_sa).wrapping_sub(c as i128);
+        let (low, c) = (low ^ out_sa).overflowing_sub(out_sa);
+        let high = (high ^ out_sa).wrapping_sub(out_sa).wrapping_sub(c as u128) as i128;
 
-        Some(Self { low, high })
+        // Check for overflow in final conversion
+        (high.is_negative() == (self.is_negative() ^ other.is_negative()))
+            .then_some(Self { low, high })
     }
 
     /// Performs wrapping division
@@ -398,6 +482,32 @@ impl i256 {
         // needless overflow.
         acc.wrapping_mul(base)
     }
+
+    /// Returns a number [`i256`] representing sign of this [`i256`].
+    ///
+    /// 0 if the number is zero
+    /// 1 if the number is positive
+    /// -1 if the number is negative
+    pub const fn signum(self) -> Self {
+        if self.is_positive() {
+            i256::ONE
+        } else if self.is_negative() {
+            i256::MINUS_ONE
+        } else {
+            i256::ZERO
+        }
+    }
+
+    /// Returns `true` if this [`i256`] is negative
+    #[inline]
+    pub const fn is_negative(self) -> bool {
+        self.high.is_negative()
+    }
+
+    /// Returns `true` if this [`i256`] is positive
+    pub const fn is_positive(self) -> bool {
+        self.high.is_positive() || self.high == 0 && self.low != 0
+    }
 }
 
 /// Temporary workaround due to lack of stable const array slicing
@@ -709,7 +819,7 @@ mod tests {
         let checked = il.checked_add(ir);
         match overflow {
             true => assert!(checked.is_none()),
-            false => assert_eq!(checked.unwrap(), actual),
+            false => assert_eq!(checked, Some(actual)),
         }
 
         // Subtraction
@@ -721,7 +831,7 @@ mod tests {
         let checked = il.checked_sub(ir);
         match overflow {
             true => assert!(checked.is_none()),
-            false => assert_eq!(checked.unwrap(), actual),
+            false => assert_eq!(checked, Some(actual), "{bl} - {br} = {expected}"),
         }
 
         // Multiplication
@@ -737,14 +847,14 @@ mod tests {
                 "{il} * {ir} = {actual} vs {bl} * {br} = {expected}"
             ),
             false => assert_eq!(
-                checked.unwrap(),
-                actual,
+                checked,
+                Some(actual),
                 "{il} * {ir} = {actual} vs {bl} * {br} = {expected}"
             ),
         }
 
         // Exponentiation
-        for exp in vec![0, 1, 3, 8, 100].into_iter() {
+        for exp in vec![0, 1, 2, 3, 8, 100].into_iter() {
             let actual = il.wrapping_pow(exp);
             let (expected, overflow) =
                 i256::from_bigint_with_overflow(bl.clone().pow(exp));
@@ -757,9 +867,9 @@ mod tests {
                     "{il} ^ {exp} = {actual} vs {bl} * {exp} = {expected}"
                 ),
                 false => assert_eq!(
-                    checked.unwrap(),
-                    actual,
-                    "{il} ^ {exp} = {actual} vs {bl} * {exp} = {expected}"
+                    checked,
+                    Some(actual),
+                    "{il} ^ {exp} = {actual} vs {bl} ^ {exp} = {expected}"
                 ),
             }
         }
@@ -791,14 +901,33 @@ mod tests {
     #[test]
     fn test_i256() {
         let candidates = [
-            i256::from_parts(0, 0),
-            i256::from_parts(0, 1),
-            i256::from_parts(0, -1),
+            i256::ZERO,
+            i256::ONE,
+            i256::MINUS_ONE,
+            i256::from_i128(2),
+            i256::from_i128(-2),
             i256::from_parts(u128::MAX, 1),
             i256::from_parts(u128::MAX, -1),
             i256::from_parts(0, 1),
             i256::from_parts(0, -1),
+            i256::from_parts(1, -1),
+            i256::from_parts(1, 1),
+            i256::from_parts(0, i128::MAX),
+            i256::from_parts(0, i128::MIN),
+            i256::from_parts(1, i128::MAX),
+            i256::from_parts(1, i128::MIN),
+            i256::from_parts(u128::MAX, i128::MIN),
             i256::from_parts(100, 32),
+            i256::MIN,
+            i256::MAX,
+            i256::MIN >> 1,
+            i256::MAX >> 1,
+            i256::ONE << 127,
+            i256::ONE << 128,
+            i256::ONE << 129,
+            i256::MINUS_ONE << 127,
+            i256::MINUS_ONE << 128,
+            i256::MINUS_ONE << 129,
         ];
 
         for il in candidates {
@@ -808,6 +937,27 @@ mod tests {
         }
     }
 
+    #[test]
+    fn test_signed_ops() {
+        // signum
+        assert_eq!(i256::from_i128(1).signum(), i256::ONE);
+        assert_eq!(i256::from_i128(0).signum(), i256::ZERO);
+        assert_eq!(i256::from_i128(-0).signum(), i256::ZERO);
+        assert_eq!(i256::from_i128(-1).signum(), i256::MINUS_ONE);
+
+        // is_positive
+        assert!(i256::from_i128(1).is_positive());
+        assert!(!i256::from_i128(0).is_positive());
+        assert!(!i256::from_i128(-0).is_positive());
+        assert!(!i256::from_i128(-1).is_positive());
+
+        // is_negative
+        assert!(!i256::from_i128(1).is_negative());
+        assert!(!i256::from_i128(0).is_negative());
+        assert!(!i256::from_i128(-0).is_negative());
+        assert!(i256::from_i128(-1).is_negative());
+    }
+
     #[test]
     #[cfg_attr(miri, ignore)]
     fn test_i256_fuzz() {
@@ -860,4 +1010,72 @@ mod tests {
         assert!(a.to_i64().is_none());
         assert!(a.to_u64().is_none());
     }
+
+    #[test]
+    fn test_i256_as_i128() {
+        let a = i256::from_i128(i128::MAX).wrapping_add(i256::from_i128(1));
+        let i128 = a.as_i128();
+        assert_eq!(i128, i128::MIN);
+
+        let a = i256::from_i128(i128::MAX).wrapping_add(i256::from_i128(2));
+        let i128 = a.as_i128();
+        assert_eq!(i128, i128::MIN + 1);
+
+        let a = i256::from_i128(i128::MIN).wrapping_sub(i256::from_i128(1));
+        let i128 = a.as_i128();
+        assert_eq!(i128, i128::MAX);
+
+        let a = i256::from_i128(i128::MIN).wrapping_sub(i256::from_i128(2));
+        let i128 = a.as_i128();
+        assert_eq!(i128, i128::MAX - 1);
+    }
+
+    #[test]
+    fn test_string_roundtrip() {
+        let roundtrip_cases = [
+            i256::ZERO,
+            i256::ONE,
+            i256::MINUS_ONE,
+            i256::from_i128(123456789),
+            i256::from_i128(-123456789),
+            i256::from_i128(i128::MIN),
+            i256::from_i128(i128::MAX),
+            i256::MIN,
+            i256::MAX,
+        ];
+        for case in roundtrip_cases {
+            let formatted = case.to_string();
+            let back: i256 = formatted.parse().unwrap();
+            assert_eq!(case, back);
+        }
+    }
+
+    #[test]
+    fn test_from_string() {
+        let cases = [
+            (
+                "000000000000000000000000000000000000000011",
+                Some(i256::from_i128(11)),
+            ),
+            (
+                "-000000000000000000000000000000000000000011",
+                Some(i256::from_i128(-11)),
+            ),
+            (
+                "-0000000000000000000000000000000000000000123456789",
+                Some(i256::from_i128(-123456789)),
+            ),
+            ("-", None),
+            ("+", None),
+            ("--1", None),
+            ("-+1", None),
+            ("000000000000000000000000000000000000000", Some(i256::ZERO)),
+            ("0000000000000000000000000000000000000000-11", None),
+            ("11-1111111111111111111111111111111111111", None),
+            ("115792089237316195423570985008687907853269984665640564039457584007913129639936", None)
+        ];
+        for (case, expected) in cases {
+            assert_eq!(i256::from_string(case), expected)
+        }
+    }
 }
