@@ -21,3 +21,6 @@ pub mod cast;
 pub use cast::*;
 pub mod display;
 pub mod parse;
+
+#[cfg(feature = "prettyprint")]
+pub mod pretty;
