@@ -183,7 +183,7 @@ fn create_array(
                 )?;
                 node_index = triple.1;
                 buffer_index = triple.2;
-                struct_arrays.push((struct_field.clone(), triple.0));
+                struct_arrays.push((struct_field.as_ref().clone(), triple.0));
             }
             let null_count = struct_node.null_count() as usize;
             let struct_array = if null_count > 0 {
@@ -263,7 +263,7 @@ fn create_array(
                 value_array.clone(),
             )?
         }
-        Union(fields, field_type_ids, mode) => {
+        Union(fields, mode) => {
             let union_node = nodes.get(node_index);
             node_index += 1;
 
@@ -292,9 +292,10 @@ fn create_array(
                 UnionMode::Sparse => None,
             };
 
-            let mut children = vec![];
+            let mut children = Vec::with_capacity(fields.len());
+            let mut ids = Vec::with_capacity(fields.len());
 
-            for field in fields {
+            for (id, field) in fields.iter() {
                 let triple = create_array(
                     nodes,
                     field,
@@ -310,11 +311,11 @@ fn create_array(
                 node_index = triple.1;
                 buffer_index = triple.2;
 
-                children.push((field.clone(), triple.0));
+                children.push((field.as_ref().clone(), triple.0));
+                ids.push(id);
             }
 
-            let array =
-                UnionArray::try_new(field_type_ids, type_ids, value_offsets, children)?;
+            let array = UnionArray::try_new(&ids, type_ids, value_offsets, children)?;
             Arc::new(array)
         }
         Null => {
@@ -418,7 +419,7 @@ fn skip_field(
             node_index += 1;
             buffer_index += 2;
         }
-        Union(fields, _field_type_ids, mode) => {
+        Union(fields, mode) => {
             node_index += 1;
             buffer_index += 1;
 
@@ -429,7 +430,7 @@ fn skip_field(
                 UnionMode::Sparse => {}
             };
 
-            for field in fields {
+            for (_, field) in fields.iter() {
                 let tuple = skip_field(field.data_type(), node_index, buffer_index)?;
 
                 node_index = tuple.0;
@@ -650,15 +651,15 @@ pub fn read_record_batch(
     // keep track of buffer and node index, the functions that create arrays mutate these
     let mut buffer_index = 0;
     let mut node_index = 0;
-    let mut arrays = vec![];
 
     let options = RecordBatchOptions::new().with_row_count(Some(batch.length() as usize));
 
     if let Some(projection) = projection {
+        let mut arrays = vec![];
         // project fields
         for (idx, field) in schema.fields().iter().enumerate() {
             // Create array for projected field
-            if projection.contains(&idx) {
+            if let Some(proj_idx) = projection.iter().position(|p| p == &idx) {
                 let triple = create_array(
                     field_nodes,
                     field,
@@ -672,7 +673,7 @@ pub fn read_record_batch(
                 )?;
                 node_index = triple.1;
                 buffer_index = triple.2;
-                arrays.push(triple.0);
+                arrays.push((proj_idx, triple.0));
             } else {
                 // Skip field.
                 // This must be called to advance `node_index` and `buffer_index`.
@@ -681,13 +682,14 @@ pub fn read_record_batch(
                 buffer_index = tuple.1;
             }
         }
-
+        arrays.sort_by_key(|t| t.0);
         RecordBatch::try_new_with_options(
             Arc::new(schema.project(projection)?),
-            arrays,
+            arrays.into_iter().map(|t| t.1).collect(),
             &options,
         )
     } else {
+        let mut arrays = vec![];
         // keep track of index as lists require more than one node
         for field in schema.fields() {
             let triple = create_array(
@@ -736,10 +738,8 @@ pub fn read_dictionary(
     let dictionary_values: ArrayRef = match first_field.data_type() {
         DataType::Dictionary(_, ref value_type) => {
             // Make a fake schema for the dictionary batch.
-            let schema = Schema {
-                fields: vec![Field::new("", value_type.as_ref().clone(), true)],
-                metadata: HashMap::new(),
-            };
+            let value = value_type.as_ref().clone();
+            let schema = Schema::new(vec![Field::new("", value, true)]);
             // Read a single column
             let record_batch = read_record_batch(
                 buf,
@@ -792,6 +792,9 @@ pub struct FileReader<R: Read + Seek> {
     /// Metadata version
     metadata_version: crate::MetadataVersion,
 
+    /// User defined metadata
+    custom_metadata: HashMap<String, String>,
+
     /// Optional projection and projected_schema
     projection: Option<(Vec<usize>, Schema)>,
 }
@@ -862,6 +865,16 @@ impl<R: Read + Seek> FileReader<R> {
         let ipc_schema = footer.schema().unwrap();
         let schema = crate::convert::fb_to_schema(ipc_schema);
 
+        let mut custom_metadata = HashMap::new();
+        if let Some(fb_custom_metadata) = footer.custom_metadata() {
+            for kv in fb_custom_metadata.into_iter() {
+                custom_metadata.insert(
+                    kv.key().unwrap().to_string(),
+                    kv.value().unwrap().to_string(),
+                );
+            }
+        }
+
         // Create an array of optional dictionary value arrays, one per field.
         let mut dictionaries_by_id = HashMap::new();
         if let Some(dictionaries) = footer.dictionaries() {
@@ -926,10 +939,16 @@ impl<R: Read + Seek> FileReader<R> {
             total_blocks,
             dictionaries_by_id,
             metadata_version: footer.version(),
+            custom_metadata,
             projection,
         })
     }
 
+    /// Return user defined customized metadata
+    pub fn custom_metadata(&self) -> &HashMap<String, String> {
+        &self.custom_metadata
+    }
+
     /// Return the number of batches in the file
     pub fn num_batches(&self) -> usize {
         self.total_blocks
@@ -1254,37 +1273,32 @@ mod tests {
     fn create_test_projection_schema() -> Schema {
         // define field types
         let list_data_type =
-            DataType::List(Box::new(Field::new("item", DataType::Int32, true)));
+            DataType::List(Arc::new(Field::new("item", DataType::Int32, true)));
 
         let fixed_size_list_data_type = DataType::FixedSizeList(
-            Box::new(Field::new("item", DataType::Int32, false)),
+            Arc::new(Field::new("item", DataType::Int32, false)),
             3,
         );
 
-        let key_type = DataType::Int8;
-        let value_type = DataType::Utf8;
-        let dict_data_type =
-            DataType::Dictionary(Box::new(key_type), Box::new(value_type));
+        let union_fields = UnionFields::new(
+            vec![0, 1],
+            vec![
+                Field::new("a", DataType::Int32, false),
+                Field::new("b", DataType::Float64, false),
+            ],
+        );
 
-        let union_fileds = vec![
-            Field::new("a", DataType::Int32, false),
-            Field::new("b", DataType::Float64, false),
-        ];
-        let union_data_type = DataType::Union(union_fileds, vec![0, 1], UnionMode::Dense);
+        let union_data_type = DataType::Union(union_fields, UnionMode::Dense);
 
-        let struct_fields = vec![
+        let struct_fields = Fields::from(vec![
             Field::new("id", DataType::Int32, false),
-            Field::new(
-                "list",
-                DataType::List(Box::new(Field::new("item", DataType::Int8, true))),
-                false,
-            ),
-        ];
+            Field::new_list("list", Field::new("item", DataType::Int8, true), false),
+        ]);
         let struct_data_type = DataType::Struct(struct_fields);
 
         let run_encoded_data_type = DataType::RunEndEncoded(
-            Box::new(Field::new("run_ends", DataType::Int16, false)),
-            Box::new(Field::new("values", DataType::Int32, true)),
+            Arc::new(Field::new("run_ends", DataType::Int16, false)),
+            Arc::new(Field::new("values", DataType::Int32, true)),
         );
 
         // define schema
@@ -1301,7 +1315,7 @@ mod tests {
             Field::new("f9", struct_data_type, false),
             Field::new("f10", run_encoded_data_type, false),
             Field::new("f11", DataType::Boolean, false),
-            Field::new("f12", dict_data_type, false),
+            Field::new_dictionary("f12", DataType::Int8, DataType::Utf8, false),
             Field::new("f13", DataType::Utf8, false),
         ])
     }
@@ -1423,6 +1437,17 @@ mod tests {
             // check the projected column equals the expected column
             assert_eq!(projected_column.as_ref(), expected_column.as_ref());
         }
+
+        {
+            // read record batch with reversed projection
+            let reader = FileReader::try_new(
+                std::io::Cursor::new(buf.clone()),
+                Some(vec![3, 2, 1]),
+            );
+            let read_batch = reader.unwrap().next().unwrap().unwrap();
+            let expected_batch = batch.project(&[3, 2, 1]).unwrap();
+            assert_eq!(read_batch, expected_batch);
+        }
     }
 
     #[test]
@@ -1516,6 +1541,25 @@ mod tests {
         reader.next().unwrap().unwrap()
     }
 
+    #[test]
+    fn test_roundtrip_with_custom_metadata() {
+        let schema = Schema::new(vec![Field::new("dummy", DataType::Float64, false)]);
+        let mut buf = Vec::new();
+        let mut writer = crate::writer::FileWriter::try_new(&mut buf, &schema).unwrap();
+        let mut test_metadata = HashMap::new();
+        test_metadata.insert("abc".to_string(), "abc".to_string());
+        test_metadata.insert("def".to_string(), "def".to_string());
+        for (k, v) in &test_metadata {
+            writer.write_metadata(k, v);
+        }
+        writer.finish().unwrap();
+        drop(writer);
+
+        let reader =
+            crate::reader::FileReader::try_new(std::io::Cursor::new(buf), None).unwrap();
+        assert_eq!(reader.custom_metadata(), &test_metadata);
+    }
+
     #[test]
     fn test_roundtrip_nested_dict() {
         let inner: DictionaryArray<Int32Type> = vec!["a", "b", "a"].into_iter().collect();
@@ -1564,7 +1608,7 @@ mod tests {
         let union1 = rb.column(0);
         let union2 = rb2.column(0);
 
-        assert_eq!(union1.data().buffers(), union2.data().buffers());
+        assert_eq!(union1, union2);
     }
 
     #[test]
@@ -1676,7 +1720,7 @@ mod tests {
             (values_field, make_array(value_dict_array.into_data())),
         ]);
         let map_data_type = DataType::Map(
-            Box::new(Field::new(
+            Arc::new(Field::new(
                 "entries",
                 entry_struct.data_type().clone(),
                 true,
@@ -1718,14 +1762,14 @@ mod tests {
         let values = StringArray::from(vec![Some("a"), None, Some("c"), None]);
         let keys = Int8Array::from_iter_values([0, 0, 1, 2, 0, 1, 3]);
         let dict_array = DictionaryArray::<Int8Type>::try_new(&keys, &values).unwrap();
-        let dict_data = dict_array.data();
+        let dict_data = dict_array.to_data();
 
         let value_offsets = Buffer::from_slice_ref(offsets);
 
         let list_data = ArrayData::builder(list_data_type)
             .len(4)
             .add_buffer(value_offsets)
-            .add_child_data(dict_data.clone())
+            .add_child_data(dict_data)
             .build()
             .unwrap();
         let list_array = GenericListArray::<OffsetSize>::from(list_data);
@@ -1748,7 +1792,7 @@ mod tests {
     #[test]
     fn test_roundtrip_stream_dict_of_list_of_dict() {
         // list
-        let list_data_type = DataType::List(Box::new(Field::new_dict(
+        let list_data_type = DataType::List(Arc::new(Field::new_dict(
             "item",
             DataType::Dictionary(Box::new(DataType::Int8), Box::new(DataType::Utf8)),
             true,
@@ -1762,7 +1806,7 @@ mod tests {
         );
 
         // large list
-        let list_data_type = DataType::LargeList(Box::new(Field::new_dict(
+        let list_data_type = DataType::LargeList(Arc::new(Field::new_dict(
             "item",
             DataType::Dictionary(Box::new(DataType::Int8), Box::new(DataType::Utf8)),
             true,
@@ -1781,10 +1825,10 @@ mod tests {
         let values = StringArray::from(vec![Some("a"), None, Some("c"), None]);
         let keys = Int8Array::from_iter_values([0, 0, 1, 2, 0, 1, 3, 1, 2]);
         let dict_array = DictionaryArray::<Int8Type>::try_new(&keys, &values).unwrap();
-        let dict_data = dict_array.data();
+        let dict_data = dict_array.to_data();
 
         let list_data_type = DataType::FixedSizeList(
-            Box::new(Field::new_dict(
+            Arc::new(Field::new_dict(
                 "item",
                 DataType::Dictionary(Box::new(DataType::Int8), Box::new(DataType::Utf8)),
                 true,
@@ -1795,7 +1839,7 @@ mod tests {
         );
         let list_data = ArrayData::builder(list_data_type)
             .len(3)
-            .add_child_data(dict_data.clone())
+            .add_child_data(dict_data)
             .build()
             .unwrap();
         let list_array = FixedSizeListArray::from(list_data);
@@ -1817,7 +1861,7 @@ mod tests {
 
     #[test]
     fn test_no_columns_batch() {
-        let schema = Arc::new(Schema::new(vec![]));
+        let schema = Arc::new(Schema::empty());
         let options = RecordBatchOptions::new()
             .with_match_field_names(true)
             .with_row_count(Some(10));
