@@ -220,15 +220,16 @@ impl IpcDataGenerator {
                 }
             }
             DataType::RunEndEncoded(_, values) => {
-                if column.data().child_data().len() != 2 {
+                let data = column.to_data();
+                if data.child_data().len() != 2 {
                     return Err(ArrowError::InvalidArgumentError(format!(
                         "The run encoded array should have exactly two child arrays. Found {}",
-                        column.data().child_data().len()
+                        data.child_data().len()
                     )));
                 }
-                // The run_ends array is not expected to be dictionoary encoded. Hence encode dictionaries
+                // The run_ends array is not expected to be dictionary encoded. Hence encode dictionaries
                 // only for values array.
-                let values_array = make_array(column.data().child_data()[1].clone());
+                let values_array = make_array(data.child_data()[1].clone());
                 self.encode_dictionaries(
                     values,
                     &values_array,
@@ -298,10 +299,10 @@ impl IpcDataGenerator {
                     write_options,
                 )?;
             }
-            DataType::Union(fields, type_ids, _) => {
+            DataType::Union(fields, _) => {
                 let union = as_union_array(column);
-                for (field, type_id) in fields.iter().zip(type_ids) {
-                    let column = union.child(*type_id);
+                for (type_id, field) in fields.iter() {
+                    let column = union.child(type_id);
                     self.encode_dictionaries(
                         field,
                         column,
@@ -330,7 +331,7 @@ impl IpcDataGenerator {
                 let dict_id = field
                     .dict_id()
                     .expect("All Dictionary types have `dict_id`");
-                let dict_data = column.data();
+                let dict_data = column.to_data();
                 let dict_values = &dict_data.child_data()[0];
 
                 let values = make_array(dict_data.child_data()[0].clone());
@@ -418,9 +419,9 @@ impl IpcDataGenerator {
             batch_compression_type.map(TryInto::try_into).transpose()?;
 
         for array in batch.columns() {
-            let array_data = array.data();
+            let array_data = array.to_data();
             offset = write_array_data(
-                array_data,
+                &array_data,
                 &mut buffers,
                 &mut arrow_data,
                 &mut nodes,
@@ -631,7 +632,7 @@ fn into_zero_offset_run_array<R: RunEndIndexType>(
 /// multiple times. Can optionally error if an update to an existing dictionary is attempted, which
 /// isn't allowed in the `FileWriter`.
 pub struct DictionaryTracker {
-    written: HashMap<i64, ArrayRef>,
+    written: HashMap<i64, ArrayData>,
     error_on_replacement: bool,
 }
 
@@ -660,18 +661,18 @@ impl DictionaryTracker {
         dict_id: i64,
         column: &ArrayRef,
     ) -> Result<bool, ArrowError> {
-        let dict_data = column.data();
+        let dict_data = column.to_data();
         let dict_values = &dict_data.child_data()[0];
 
         // If a dictionary with this id was already emitted, check if it was the same.
         if let Some(last) = self.written.get(&dict_id) {
-            if ArrayData::ptr_eq(&last.data().child_data()[0], dict_values) {
+            if ArrayData::ptr_eq(&last.child_data()[0], dict_values) {
                 // Same dictionary values => no need to emit it again
                 return Ok(false);
             }
             if self.error_on_replacement {
                 // If error on replacement perform a logical comparison
-                if last.data().child_data()[0] == *dict_values {
+                if last.child_data()[0] == *dict_values {
                     // Same dictionary values => no need to emit it again
                     return Ok(false);
                 }
@@ -684,7 +685,7 @@ impl DictionaryTracker {
             }
         }
 
-        self.written.insert(dict_id, column.clone());
+        self.written.insert(dict_id, dict_data);
         Ok(true)
     }
 }
@@ -706,6 +707,8 @@ pub struct FileWriter<W: Write> {
     finished: bool,
     /// Keeps track of dictionaries that have been written
     dictionary_tracker: DictionaryTracker,
+    /// User level customized metadata
+    custom_metadata: HashMap<String, String>,
 
     data_gen: IpcDataGenerator,
 }
@@ -742,10 +745,15 @@ impl<W: Write> FileWriter<W> {
             record_blocks: vec![],
             finished: false,
             dictionary_tracker: DictionaryTracker::new(true),
+            custom_metadata: HashMap::new(),
             data_gen,
         })
     }
 
+    pub fn write_metadata(&mut self, key: impl Into<String>, value: impl Into<String>) {
+        self.custom_metadata.insert(key.into(), value.into());
+    }
+
     /// Write a record batch to the file
     pub fn write(&mut self, batch: &RecordBatch) -> Result<(), ArrowError> {
         if self.finished {
@@ -798,6 +806,8 @@ impl<W: Write> FileWriter<W> {
         let dictionaries = fbb.create_vector(&self.dictionary_blocks);
         let record_batches = fbb.create_vector(&self.record_blocks);
         let schema = crate::convert::schema_to_fb_offset(&mut fbb, &self.schema);
+        let fb_custom_metadata = (!self.custom_metadata.is_empty())
+            .then(|| crate::convert::metadata_to_fb(&mut fbb, &self.custom_metadata));
 
         let root = {
             let mut footer_builder = crate::FooterBuilder::new(&mut fbb);
@@ -805,6 +815,9 @@ impl<W: Write> FileWriter<W> {
             footer_builder.add_schema(schema);
             footer_builder.add_dictionaries(dictionaries);
             footer_builder.add_recordBatches(record_batches);
+            if let Some(fb_custom_metadata) = fb_custom_metadata {
+                footer_builder.add_custom_metadata(fb_custom_metadata);
+            }
             footer_builder.finish()
         };
         fbb.finish(root, None);
@@ -937,7 +950,7 @@ impl<W: Write> StreamWriter<W> {
     ///     255, 255, 255, 255,   0,   0,   0,   0
     /// ];
     ///
-    /// let schema = Schema::new(vec![]);
+    /// let schema = Schema::empty();
     /// let buffer: Vec<u8> = Vec::new();
     /// let options = IpcWriteOptions::try_new(8, false, MetadataVersion::V5)?;
     /// let stream_writer = StreamWriter::try_new_with_options(buffer, &schema, options)?;
@@ -1069,7 +1082,7 @@ fn has_validity_bitmap(data_type: &DataType, write_options: &IpcWriteOptions) ->
     } else {
         !matches!(
             data_type,
-            DataType::Null | DataType::Union(_, _, _) | DataType::RunEndEncoded(_, _)
+            DataType::Null | DataType::Union(_, _) | DataType::RunEndEncoded(_, _)
         )
     }
 }
@@ -1778,17 +1791,14 @@ mod tests {
     }
 
     fn write_union_file(options: IpcWriteOptions) {
-        let schema = Schema::new(vec![Field::new(
+        let schema = Schema::new(vec![Field::new_union(
             "union",
-            DataType::Union(
-                vec![
-                    Field::new("a", DataType::Int32, false),
-                    Field::new("c", DataType::Float64, false),
-                ],
-                vec![0, 1],
-                UnionMode::Sparse,
-            ),
-            true,
+            vec![0, 1],
+            vec![
+                Field::new("a", DataType::Int32, false),
+                Field::new("c", DataType::Float64, false),
+            ],
+            UnionMode::Sparse,
         )]);
         let mut builder = UnionBuilder::with_capacity_sparse(5);
         builder.append::<Int32Type>("a", 1).unwrap();
@@ -2024,15 +2034,11 @@ mod tests {
         );
 
         let sliced = array.slice(1, 2);
-        let read_sliced: &UInt32Array = sliced.as_primitive();
-        assert_eq!(
-            vec![Some(2), Some(3)],
-            read_sliced.iter().collect::<Vec<_>>()
-        );
+        assert_eq!(vec![Some(2), Some(3)], sliced.iter().collect::<Vec<_>>());
 
         let batch = RecordBatch::try_new(
             Arc::new(Schema::new(vec![Field::new("a", DataType::UInt32, true)])),
-            vec![sliced],
+            vec![Arc::new(sliced)],
         )
         .expect("new batch");
 
