@@ -15,7 +15,9 @@
 // specific language governing permissions and limitations
 // under the License.
 
+use crate::raw::serializer::TapeSerializer;
 use arrow_schema::ArrowError;
+use serde::Serialize;
 use std::fmt::{Display, Formatter};
 
 /// We decode JSON to a flattened tape representation,
@@ -452,6 +454,27 @@ impl TapeDecoder {
         Ok(buf.len() - iter.len())
     }
 
+    /// Writes any type that implements [`Serialize`] into this [`TapeDecoder`]
+    pub fn serialize<S: Serialize>(&mut self, rows: &[S]) -> Result<(), ArrowError> {
+        if let Some(b) = self.stack.last() {
+            return Err(ArrowError::JsonError(format!(
+                "Cannot serialize to tape containing partial decode state {}",
+                b.as_str()
+            )));
+        }
+
+        let mut serializer =
+            TapeSerializer::new(&mut self.elements, &mut self.bytes, &mut self.offsets);
+
+        rows.iter()
+            .try_for_each(|row| row.serialize(&mut serializer))
+            .map_err(|e| ArrowError::JsonError(e.to_string()))?;
+
+        self.num_rows += rows.len();
+
+        Ok(())
+    }
+
     /// Finishes the current [`Tape`]
     pub fn finish(&self) -> Result<Tape<'_>, ArrowError> {
         if let Some(b) = self.stack.last() {
