@@ -23,6 +23,8 @@ mod error;
 pub use error::*;
 mod field;
 pub use field::*;
+mod fields;
+pub use fields::*;
 mod schema;
 pub use schema::*;
 use std::ops;
