@@ -22,12 +22,11 @@ use arrow::array::{
     Array, Decimal128Array, Decimal128Builder, Decimal256Array, Decimal256Builder,
 };
 use criterion::Criterion;
-use num::BigInt;
 use rand::Rng;
 
 extern crate arrow;
 
-use arrow::util::decimal::Decimal256;
+use arrow_buffer::i256;
 
 fn validate_decimal128_array(array: Decimal128Array) {
     array.with_precision_and_scale(35, 0).unwrap();
@@ -40,13 +39,14 @@ fn validate_decimal256_array(array: Decimal256Array) {
 fn validate_decimal128_benchmark(c: &mut Criterion) {
     let mut rng = rand::thread_rng();
     let size: i128 = 20000;
-    let mut decimal_builder = Decimal128Builder::with_capacity(size as usize, 38, 0);
+    let mut decimal_builder = Decimal128Builder::with_capacity(size as usize);
     for _ in 0..size {
-        decimal_builder
-            .append_value(rng.gen_range::<i128, _>(0..999999999999))
-            .unwrap();
+        decimal_builder.append_value(rng.gen_range::<i128, _>(0..999999999999));
     }
-    let decimal_array = decimal_builder.finish();
+    let decimal_array = decimal_builder
+        .finish()
+        .with_precision_and_scale(38, 0)
+        .unwrap();
     let data = decimal_array.into_data();
     c.bench_function("validate_decimal128_array 20000", |b| {
         b.iter(|| {
@@ -59,13 +59,16 @@ fn validate_decimal128_benchmark(c: &mut Criterion) {
 fn validate_decimal256_benchmark(c: &mut Criterion) {
     let mut rng = rand::thread_rng();
     let size: i128 = 20000;
-    let mut decimal_builder = Decimal256Builder::with_capacity(size as usize, 76, 0);
+    let mut decimal_builder = Decimal256Builder::with_capacity(size as usize);
     for _ in 0..size {
         let v = rng.gen_range::<i128, _>(0..999999999999999);
-        let decimal = Decimal256::from_big_int(&BigInt::from(v), 76, 0).unwrap();
-        decimal_builder.append_value(&decimal).unwrap();
+        let decimal = i256::from_i128(v);
+        decimal_builder.append_value(decimal);
     }
-    let decimal_array256_data = decimal_builder.finish();
+    let decimal_array256_data = decimal_builder
+        .finish()
+        .with_precision_and_scale(76, 0)
+        .unwrap();
     let data = decimal_array256_data.into_data();
     c.bench_function("validate_decimal256_array 20000", |b| {
         b.iter(|| {
